/*
 * Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef WORKER_GRAPH_H_
#define WORKER_GRAPH_H_

#include "ucp/api/ucp.h"

#include <doca_urom.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Loopback task callback function, will be called once the loopback task is finished
 *
 * @result [in]: task result
 * @cookie [in]: user cookie
 * @data [in]: loopback data
 */
typedef void (*urom_graph_loopback_finished)(doca_error_t result, union doca_data cookie, uint64_t data);

/*
 * Creates send sandbox tag task
 *
 * @worker_ctx [in]: DOCA UROM worker context
 * @cookie [in]: user cookie
 * @data [in]: loopback data
 * @cb [in]: program callback to call once the task is finished
 * @return: DOCA_SUCCESS on success and DOCA_ERROR otherwise
 */
doca_error_t urom_graph_task_loopback(struct doca_urom_worker *worker_ctx,
				      union doca_data cookie,
				      uint64_t data,
				      urom_graph_loopback_finished cb);

/*
 *
 * This method inits graph plugin.
 *
 * @plugin_id [in]: plugin id
 * @version [in]: plugin version on DPU side
 * @return: DOCA_SUCCESS on success and DOCA_ERROR otherwise
 */
doca_error_t urom_graph_init(uint64_t plugin_id, uint64_t version);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* WORKER_GRAPH_H_ */
